@@ -1,3 +1,19 @@
+# Copyright (C) 2022 Brodie Gaslam
+# 
+# This file is part of "unitizer"
+# 
+# This program is free software: you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation, either version 2 of the License, or
+# (at your option) any later version.
+# 
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+# 
+# Go to <https://www.r-project.org/Licenses/GPL-2> for a copy of the license.
+
 #' @include item.R
 #' @include item.sub.R
 #' @include section.R
@@ -88,6 +104,13 @@ setClassUnion(
 # @slot sections a list of \code{\link{unitizerSection-class}}
 # @slot section.map a map of every item in \code{items.new} to a section
 # @slot changes contains summary data on the test results
+# @slot upgraded.from character(1L) whether the unitizer was upgraded from load
+#   and from what version, "" if it was not upgraded.
+# @slot best.name character(1L) a friendlier name derived from test and unitizer
+#   locations.
+# @slot show.progress integer(1L) level of chattiness in progress reporting,
+#   carried here as otherwise no way to feed it through the `+` method when
+#   adding tests.
 
 setClass(
   "unitizer",
@@ -148,7 +171,14 @@ setClass(
 
     changes="unitizerChanges",       # Summary of user changes
     res.data="data.frameOrNULL",     # details of test evaluation and user review
-    bookmark="unitizerBrowseBookmarkOrNULL"  # used for re-eval navigation
+    bookmark="unitizerBrowseBookmarkOrNULL",  # used for re-eval navigation
+
+    # fields to support >1.4.14 functionality
+
+    upgraded.from="character",       # was upgraded on load
+    best.name="character",
+    show.progress="integer"
+
   ),
   prototype(
     version=as.character(packageVersion("unitizer")),
@@ -161,7 +191,9 @@ setClass(
     updated=FALSE,
     updated.at.least.once=FALSE,
     bookmark=NULL,
-    global=unitizerGlobal$new(enable.which=character())  # dummy so tests will run
+    global=unitizerGlobal$new(enable.which=character()),  # dummy so tests will run
+    upgraded.from="",
+    show.progress=0L
   ),
   validity=function(object) {
     if(!is.object(object@id) && is.character(object@id)) { # default id format
